from .objprope import get_class, get_class_name, get_methods_on_class

from .objverify import is_type_of, is_dict, is_lazy_url, is_soup, is_response
from .objverify import has_attr_of_type, has_soup, has_response, has_headers

from .objextend import validate_attribute_name, add_attribute
from .objextend import make_get_has_header_methods_for_obj_with_response

from .urls import LazyUrl
